@@ -1,32 +1,9 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2018 Ilya Bizyaev <bizyaev@zoho.com>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2021 Jonah Brüchert <jbb@kaidan.im>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "AvatarFileStorage.h"
 // Qt
@@ -63,7 +40,7 @@ AvatarFileStorage::AvatarFileStorage(QObject *parent) : QObject(parent)
 			QString line = stream.readLine();
 			while (!line.isNull()) {
 				// get hash and jid from line (seperated by a blank)
-				QStringList list = line.split(' ', Qt::SkipEmptyParts);
+				QStringList list = line.split(QLatin1Char(' '), Qt::SkipEmptyParts);
 
 				// set the hash for the jid
 				m_jidAvatarMap[list.at(1)] = list.at(0);
@@ -83,8 +60,8 @@ AvatarFileStorage::AddAvatarResult AvatarFileStorage::addAvatar(const QString &j
 	AddAvatarResult result;
 
 	// generate a hexadecimal hash of the raw avatar
-	result.hash = QString(QCryptographicHash::hash(avatar, QCryptographicHash::Sha1).toHex());
-	QString oldHash = m_jidAvatarMap[jid];
+	result.hash = QString::fromUtf8(QCryptographicHash::hash(avatar, QCryptographicHash::Sha1).toHex());
+	QString oldHash = QString::fromUtf8(m_jidAvatarMap[jid].toUtf8());
 
 	// set the new hash and the `hasChanged` tag
 	if (oldHash != result.hash) {
@@ -99,7 +76,7 @@ AvatarFileStorage::AddAvatarResult AvatarFileStorage::addAvatar(const QString &j
 	// only update GUI, if avatar really has changed
 	if (hasAvatarHash(result.hash)) {
 		if (result.hasChanged)
-			emit avatarIdsChanged();
+			Q_EMIT avatarIdsChanged();
 		return result;
 	}
 
@@ -118,7 +95,7 @@ AvatarFileStorage::AddAvatarResult AvatarFileStorage::addAvatar(const QString &j
 	// mark that the avatar is new
 	result.newWritten = true;
 
-	emit avatarIdsChanged();
+	Q_EMIT avatarIdsChanged();
 	return result;
 }
 
@@ -135,7 +112,7 @@ void AvatarFileStorage::clearAvatar(const QString &jid)
 	m_jidAvatarMap.remove(jid);
 	saveAvatarsFile();
 	cleanUp(oldHash);
-	emit avatarIdsChanged();
+	Q_EMIT avatarIdsChanged();
 }
 
 void AvatarFileStorage::cleanUp(QString &oldHash)
