@@ -1,32 +1,7 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2020 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "DataFormModel.h"
 // Qt
@@ -63,7 +38,8 @@ QVariant DataFormModel::data(const QModelIndex &index, int role) const
 	if (!index.isValid() || !hasIndex(index.row(), index.column(), index.parent()))
 		return {};
 
-	const QXmppDataForm::Field &field = m_form.fields().at(index.row());
+	auto fields = m_form.fields();
+	const auto &field = fields.at(index.row());
 
 	switch(role) {
 	case Key:
@@ -116,7 +92,7 @@ bool DataFormModel::setData(const QModelIndex &index, const QVariant &value, int
 	}
 
 	m_form.fields()[index.row()] = field;
-	emit dataChanged(index, index, QVector<int>() << role);
+	Q_EMIT dataChanged(index, index, QVector<int>() << role);
 	return true;
 }
 
@@ -144,7 +120,7 @@ void DataFormModel::setForm(const QXmppDataForm &form)
 	m_form = form;
 	endResetModel();
 
-	emit formChanged();
+	Q_EMIT formChanged();
 }
 
 QString DataFormModel::instructions() const
@@ -162,7 +138,7 @@ QString DataFormModel::mediaSourceUri(const QXmppDataForm::Field &field) const
 		// Prefer Bits of Binary URIs.
 		// In most cases, the data has been received already then.
 		if (QXmppBitsOfBinaryContentId::isBitsOfBinaryContentId(mediaSourceUri, true))
-			return QStringLiteral("image://%1/%2").arg(BITS_OF_BINARY_IMAGE_PROVIDER_NAME, mediaSourceUri);
+			return QStringLiteral("image://%1/%2").arg(QStringLiteral(BITS_OF_BINARY_IMAGE_PROVIDER_NAME), mediaSourceUri);
 	}
 
 	return mediaSourceUri;
