@@ -1,32 +1,13 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2019 Xavier <xavi@delape.net>
+// SPDX-FileCopyrightText: 2020 Yury Gubich <blue@macaw.me>
+// SPDX-FileCopyrightText: 2022 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2022 Bhavy Airi <airiragahv@gmail.com>
+// SPDX-FileCopyrightText: 2023 Filipe Azevedo <pasnox@gmail.com>
+// SPDX-FileCopyrightText: 2023 Tibor Csötönyi <work@taibsu.de>
+// SPDX-FileCopyrightText: 2024 Filipe Azevedo <pasnox@gmail.com>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
@@ -57,6 +38,9 @@ public:
 	 */
 	void createTables();
 
+	// used in unit tests
+	void createV3Database();
+
 	/// Transaction on random thread from the thread pool (should be replaced in the
 	/// future).
 	void startTransaction();
@@ -129,6 +113,28 @@ private:
 	void convertDatabaseToV25();
 	void convertDatabaseToV26();
 	void convertDatabaseToV27();
+	void convertDatabaseToV28();
+	void convertDatabaseToV29();
+	void convertDatabaseToV30();
+	void convertDatabaseToV31();
+	void convertDatabaseToV32();
+	void convertDatabaseToV33();
+	void convertDatabaseToV34();
+	void convertDatabaseToV35();
+	void convertDatabaseToV36();
+	void convertDatabaseToV37();
+	void convertDatabaseToV38();
+	void convertDatabaseToV39();
+	void convertDatabaseToV40();
+	void convertDatabaseToV41();
+	void convertDatabaseToV42();
+	void convertDatabaseToV43();
+	void convertDatabaseToV44();
+	void convertDatabaseToV45();
+	void convertDatabaseToV46();
+	void convertDatabaseToV47();
 
 	std::unique_ptr<DatabasePrivate> d;
+
+	friend class DatabaseTest;
 };
