@@ -1,3 +1,11 @@
+// SPDX-FileCopyrightText: 2017 Konstantinos Sideris <siderisk@auth.gr>
+// SPDX-FileCopyrightText: 2018 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2019 Filipe Azevedo <pasnox@gmail.com>
+// SPDX-FileCopyrightText: 2019 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 #pragma once
 
 #include <QAbstractListModel>
@@ -82,13 +90,11 @@ public:
 
 	bool hasFavoriteEmojis() const;
 
-public slots:
-	void addFavoriteEmoji(int proxyRow);
+	Q_INVOKABLE void addFavoriteEmoji(int proxyRow);
 
-signals:
-	void groupChanged();
-	void filterChanged();
-	void hasFavoriteEmojisChanged();
+	Q_SIGNAL void groupChanged();
+	Q_SIGNAL void filterChanged();
+	Q_SIGNAL void hasFavoriteEmojisChanged();
 
 protected:
 	bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override;
