@@ -1,36 +1,16 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2019 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2020 Mathis Brüchert <mbblp@protonmail.ch>
+// SPDX-FileCopyrightText: 2023 Filipe Azevedo <pasnox@gmail.com>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #ifndef GLOBALS_H
 #define GLOBALS_H
 
+#include <GlobalsGen.h>
+
 #include <QLatin1String>
 
 // Kaidan settings
@@ -38,15 +18,20 @@
 #define KAIDAN_SETTINGS_AUTH_JID "auth/jid"
 #define KAIDAN_SETTINGS_AUTH_JID_RESOURCE_PREFIX "auth/jidResourcePrefix"
 #define KAIDAN_SETTINGS_AUTH_PASSWD "auth/password"
+#define KAIDAN_SETTINGS_AUTH_CREDENTIALS "auth/credentials"
 #define KAIDAN_SETTINGS_AUTH_HOST "auth/host"
 #define KAIDAN_SETTINGS_AUTH_PORT "auth/port"
+#define KAIDAN_SETTINGS_AUTH_TLS_ERRORS_IGNORED "auth/tlsErrorsIgnored"
+#define KAIDAN_SETTINGS_AUTH_TLS_REQUIREMENT "auth/tlsRequirement"
 #define KAIDAN_SETTINGS_AUTH_PASSWD_VISIBILITY "auth/passwordVisibility"
+#define KAIDAN_SETTINGS_AUTH_DEVICE_ID "auth/userAgentDeviceId"
 #define KAIDAN_SETTINGS_ENCRYPTION "encryption"
-#define KAIDAN_SETTINGS_NOTIFICATIONS_MUTED "muted/"
 #define KAIDAN_SETTINGS_FAVORITE_EMOJIS "emojis/favorites"
 #define KAIDAN_SETTINGS_WINDOW_POSITION "window/position"
 #define KAIDAN_SETTINGS_WINDOW_SIZE "window/size"
-#define KAIDAN_SETTINGS_HELP_VISIBILITY_QR_CODE_PAGE "helpVisibility/qrCodePage"
+#define KAIDAN_SETTINGS_AUTOMATIC_MEDIA_DOWNLOADS_RULE "media/automaticDownloadsRule"
+#define KAIDAN_SETTINGS_EXPLANATION_VISIBILITY_CONTACT_ADDITION_QR_CODE_PAGE "explanationVisibility/contactAdditionQrCodePage"
+#define KAIDAN_SETTINGS_EXPLANATION_VISIBILITY_KEY_AUTHENTICATION_PAGE "explanationVisibility/keyAuthenticationPage"
 
 #define KAIDAN_JID_RESOURCE_DEFAULT_PREFIX APPLICATION_DISPLAY_NAME
 
@@ -56,21 +41,26 @@
 #define ISSUE_TRACKING_URL "https://invent.kde.org/network/kaidan/issues"
 #define DONATION_URL "https://liberapay.com/kaidan"
 #define MASTODON_URL "https://fosstodon.org/@kaidan"
+inline constexpr QStringView PROVIDER_DETAILS_URL = u"https://providers.xmpp.net/provider/%1/";
 
 constexpr auto MESSAGE_MAX_CHARS = 1e4;
 
 // SQL
 #define DB_FILE_BASE_NAME "kaidan"
 #define DB_TABLE_INFO "dbinfo"
+#define DB_TABLE_ACCOUNTS "accounts"
 #define DB_TABLE_ROSTER "roster"
+#define DB_TABLE_ROSTER_GROUPS "rosterGroups"
 #define DB_TABLE_MESSAGES "messages"
 #define DB_VIEW_CHAT_MESSAGES "chatMessages"
 #define DB_VIEW_DRAFT_MESSAGES "draftMessages"
+#define DB_TABLE_GROUP_CHAT_USERS "groupChatUsers"
 #define DB_TABLE_FILES "files"
 #define DB_TABLE_FILE_HASHES "fileHashes"
 #define DB_TABLE_FILE_HTTP_SOURCES "fileHttpSources"
 #define DB_TABLE_FILE_ENCRYPTED_SOURCES "fileEncryptedSources"
 #define DB_TABLE_MESSAGE_REACTIONS "messageReactions"
+#define DB_TABLE_BLOCKED "blocked"
 #define DB_TABLE_TRUST_SECURITY_POLICIES "trustSecurityPolicies"
 #define DB_TABLE_TRUST_OWN_KEYS "trustOwnKeys"
 #define DB_TABLE_TRUST_KEYS "trustKeys"
@@ -79,7 +69,9 @@ constexpr auto MESSAGE_MAX_CHARS = 1e4;
 #define DB_TABLE_OMEMO_OWN_DEVICES "omemoDevicesOwn"
 #define DB_TABLE_OMEMO_PRE_KEY_PAIRS "omemoPreKeyPairs"
 #define DB_TABLE_OMEMO_SIGNED_PRE_KEY_PAIRS "omemoPreKeyPairsSigned"
+#define DB_TABLE_ROSTER_GROUPS "rosterGroups"
 #define DB_QUERY_LIMIT_MESSAGES 20
+#define DB_QUERY_LIMIT_GROUP_CHAT_USERS 80
 
 //
 // Credential generation
@@ -111,6 +103,14 @@ constexpr auto GENERATED_PASSWORD_LENGTH_UPPER_BOUND = 30;
  */
 #define PROVIDER_LIST_FILE_PATH QStringLiteral(":/data/providers.json")
 
+/**
+ * Path of the JSON provider completion list file
+ */
+#define PROVIDER_COMPLETION_LIST_FILE_PATH QStringLiteral(":/data/providers-completion.json")
+
+constexpr QStringView DEFAULT_LANGUAGE_CODE = u"EN";
+constexpr QStringView DEFAULT_COUNTRY_CODE = u"US";
+
 /**
  * Number of providers required in a country so that only providers from that country are
  * randomly selected.
@@ -124,9 +124,21 @@ constexpr auto PROVIDER_LIST_MIN_PROVIDERS_FROM_COUNTRY = 2;
 
 // JPEG export quality used when saving images lossy (e.g. when saving images from clipboard)
 constexpr auto JPEG_EXPORT_QUALITY = 85;
-// Maximum file size for reading files just to generate an image thumbnail.
-constexpr auto THUMBNAIL_GENERATION_MAX_FILE_SIZE = 10 * 1024 * 1024;
 // Width and height of generated file thumbnails.
-constexpr auto THUMBNAIL_PIXEL_SIZE = 50;
+constexpr auto THUMBNAIL_EDGE_PIXEL_COUNT = 15;
+// Width and height of generated video thumbnails.
+constexpr auto VIDEO_THUMBNAIL_EDGE_PIXEL_COUNT = 500;
+
+// Count of encryption key ID characters that are grouped to be displayed for better readability
+constexpr int ENCRYPTION_KEY_ID_CHARACTER_GROUP_SIZE = 8;
+
+// Separator between grouped encryption key ID characters to be displayed for better readability
+constexpr QStringView ENCRYPTION_KEY_ID_CHARACTER_GROUP_SEPARATOR = u" ";
+
+// XMPP
+// TODO: Find a solution to not define namespaces in Kaidan
+inline constexpr QStringView XMLNS_SFS = u"urn:xmpp:sfs:0";
+inline constexpr QStringView XMLNS_MESSAGE_REPLIES = u"urn:xmpp:reply:0";
+const auto XMLNS_OMEMO_2 = QStringLiteral("urn:xmpp:omemo:2");
 
 #endif // GLOBALS_H
