@@ -1,40 +1,18 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2019 Filipe Azevedo <pasnox@gmail.com>
+// SPDX-FileCopyrightText: 2019 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2021 Linus Jahn <lnj@kaidan.im>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "MediaRecorder.h"
 #include "Kaidan.h"
 
+#include <QDir>
 #include <QUrl>
 #include <QFile>
 #include <QSettings>
 
+#include "MediaUtils.h"
 #include "Settings.h"
 
 /*
@@ -609,7 +587,7 @@ void MediaRecorder::setMediaSettings(const MediaSettings &settings)
 			break;
 		}
 
-		emit mediaSettingsChanged();
+		Q_EMIT mediaSettingsChanged();
 	}
 }
 
@@ -636,7 +614,7 @@ void MediaRecorder::setImageEncoderSettings(const ImageEncoderSettings &settings
 
 			if (!m_initializing) {
 				m_imageCapturer->setEncodingSettings(m_imageEncoderSettings.toQImageEncoderSettings());
-				emit imageEncoderSettingsChanged();
+				Q_EMIT imageEncoderSettingsChanged();
 			}
 		}
 
@@ -679,11 +657,11 @@ void MediaRecorder::setAudioEncoderSettings(const AudioEncoderSettings &settings
 					m_videoRecorder->setAudioSettings(m_audioEncoderSettings.toQAudioEncoderSettings());
 				}
 
-				emit audioEncoderSettingsChanged();
+				Q_EMIT audioEncoderSettingsChanged();
 
 				// Changing audio settings does not trigger ready changed.
 				if (m_type == MediaRecorder::Type::Audio) {
-					emit readyChanged();
+					Q_EMIT readyChanged();
 				}
 			}
 		}
@@ -720,7 +698,7 @@ void MediaRecorder::setVideoEncoderSettings(const VideoEncoderSettings &settings
 
 			if (!m_initializing) {
 				m_videoRecorder->setVideoSettings(m_videoEncoderSettings.toQVideoEncoderSettings());
-				emit videoEncoderSettingsChanged();
+				Q_EMIT videoEncoderSettingsChanged();
 			}
 		}
 
@@ -831,13 +809,14 @@ void MediaRecorder::record()
 		m_mediaSettings.dumpProperties("Capture");
 		m_imageEncoderSettings.dumpProperties("Capture");
 #endif
-		m_imageCapturer->capture();
+		m_imageCapturer->capture(MediaUtils::newImageFilePath());
 		break;
 	case MediaRecorder::Type::Audio:
 #if ENABLE_DEBUG
 		m_mediaSettings.dumpProperties("Capture");
 		m_audioEncoderSettings.dumpProperties("Capture");
 #endif
+		m_audioRecorder->setOutputLocation(MediaUtils::newAudioFilePath());
 		m_audioRecorder->record();
 		break;
 	case MediaRecorder::Type::Video:
@@ -846,6 +825,7 @@ void MediaRecorder::record()
 		m_audioEncoderSettings.dumpProperties("Capture");
 		m_videoEncoderSettings.dumpProperties("Capture");
 #endif
+		m_videoRecorder->setOutputLocation(MediaUtils::newVideoFilePath());
 		m_videoRecorder->record();
 		break;
 	}
@@ -1128,9 +1108,9 @@ void MediaRecorder::setupRecorder(MediaRecorder::Type type)
 
 	switch (m_type) {
 	case MediaRecorder::Type::Invalid:
-		emit imageEncoderSettingsChanged();
-		emit audioEncoderSettingsChanged();
-		emit videoEncoderSettingsChanged();
+		Q_EMIT imageEncoderSettingsChanged();
+		Q_EMIT audioEncoderSettingsChanged();
+		Q_EMIT videoEncoderSettingsChanged();
 		break;
 	case MediaRecorder::Type::Image:
 		setupCamera();
@@ -1145,16 +1125,16 @@ void MediaRecorder::setupRecorder(MediaRecorder::Type type)
 		break;
 	}
 
-	emit typeChanged();
-	emit availabilityStatusChanged();
-	emit stateChanged();
-	emit statusChanged();
-	emit readyChanged();
-	emit errorChanged();
-	emit actualLocationChanged();
-	emit durationChanged();
-	emit mutedChanged();
-	emit volumeChanged();
+	Q_EMIT typeChanged();
+	Q_EMIT availabilityStatusChanged();
+	Q_EMIT stateChanged();
+	Q_EMIT statusChanged();
+	Q_EMIT readyChanged();
+	Q_EMIT errorChanged();
+	Q_EMIT actualLocationChanged();
+	Q_EMIT durationChanged();
+	Q_EMIT mutedChanged();
+	Q_EMIT volumeChanged();
 
 	if (m_camera && m_camera->state() != QCamera::State::ActiveState) {
 		m_camera->start();
@@ -1171,7 +1151,7 @@ void MediaRecorder::deleteActualLocation()
 
 		if (file.exists()) {
 			if (file.remove()) {
-				emit actualLocationChanged();
+				Q_EMIT actualLocationChanged();
 			} else {
 				qWarning("Can not delete record '%s'", qUtf8Printable(filePath));
 			}
