@@ -1,49 +1,46 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2019 Filipe Azevedo <pasnox@gmail.com>
+// SPDX-FileCopyrightText: 2020 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "MediaUtils.h"
-#include "Globals.h"
-#include "FutureUtils.h"
 
+// Qt
+#include <QBuffer>
+#include <QClipboard>
+#include <QDir>
 #include <QFileInfo>
+#include <QGuiApplication>
+#include <QImage>
+#include <QImageReader>
+#include <QMimeType>
+#include <QPixmap>
 #include <QRegularExpression>
 #include <QTime>
 #include <QUrl>
-#include <QBuffer>
-#include <QMimeType>
-#include <QImage>
-#include <QPixmap>
-
 // KDE
 #include <KIO/PreviewJob>
 #include <KFileItem>
+// Kaidan
+#include "FutureUtils.h"
+#include "SystemUtils.h"
+
+const auto IMAGE_FILE_EXTENSION = QStringLiteral(".jpg");
+const auto AUDIO_FILE_EXTENSION = QStringLiteral(".mp3");
+const auto VIDEO_FILE_EXTENSION = QStringLiteral(".mp4");
+
+QString fileNameTimestamp(const QDateTime &dateTime)
+{
+	return dateTime.toString(u"yyyyMMdd_hhmmss");
+}
+
+QString newFilePath(const QString &directoryPath, const QString &fileExtension)
+{
+	QDir().mkdir(directoryPath);
+	return { directoryPath + QDir::separator() + fileNameTimestamp(QDateTime::currentDateTimeUtc()) + fileExtension };
+}
 
 static QList<QMimeType> mimeTypes(const QList<QMimeType> &mimeTypes, const QString &parent);
 
@@ -185,9 +182,44 @@ bool MediaUtils::localFileAvailable(const QString &filePath)
 	return QFile::exists(filePath);
 }
 
-QUrl MediaUtils::fromLocalFile(const QString &filePath)
+QUrl MediaUtils::urlFromClipboard()
 {
-	return QUrl::fromLocalFile(filePath);
+	const auto clipboardContent = QGuiApplication::clipboard()->text();
+
+	if (clipboardContent.startsWith("file:///")) {
+		return QUrl { clipboardContent };
+	}
+
+	if (clipboardContent.startsWith("/")) {
+		return QUrl::fromLocalFile(clipboardContent);
+	}
+
+	return {};
+}
+
+void MediaUtils::deleteFile(const QUrl &url)
+{
+	QFile::remove(url.toLocalFile());
+}
+
+QString MediaUtils::localFilePath(const QUrl &localFileUrl)
+{
+	return localFileUrl.toLocalFile();
+}
+
+QUrl MediaUtils::localFileUrl(const QString &localFilePath)
+{
+	return QUrl::fromLocalFile(localFilePath);
+}
+
+QUrl MediaUtils::localFileDirectoryUrl(const QUrl &localFileUrl)
+{
+	return QUrl::fromLocalFile(QFileInfo(localFilePath(localFileUrl)).absolutePath());
+}
+
+bool MediaUtils::imageValid(const QImage &image)
+{
+	return !image.isNull();
 }
 
 QMimeType MediaUtils::mimeType(const QString &filePath)
@@ -244,6 +276,21 @@ QString MediaUtils::iconName(const QUrl &url)
 	return iconName(url.fileName());
 }
 
+QString MediaUtils::newAudioFilePath()
+{
+	return newFilePath(SystemUtils::audioDirectory(), AUDIO_FILE_EXTENSION);
+}
+
+QString MediaUtils::newImageFilePath()
+{
+	return newFilePath(SystemUtils::imageDirectory(), IMAGE_FILE_EXTENSION);
+}
+
+QString MediaUtils::newVideoFilePath()
+{
+	return newFilePath(SystemUtils::videoDirectory(), VIDEO_FILE_EXTENSION);
+}
+
 QString MediaUtils::mediaTypeName(Enums::MessageType mediaType)
 {
 	switch (mediaType) {
@@ -514,22 +561,19 @@ QFuture<std::shared_ptr<QXmppFileSharingManager::MetadataGeneratorResult>> Media
 	// create job
 	auto *job = new KIO::PreviewJob(
 		{ KFileItem(QUrl::fromLocalFile(file->fileName())) },
-		QSize(THUMBNAIL_PIXEL_SIZE, THUMBNAIL_PIXEL_SIZE),
+		QSize(THUMBNAIL_EDGE_PIXEL_COUNT, THUMBNAIL_EDGE_PIXEL_COUNT),
 		&allPlugins
 	);
 	job->setAutoDelete(true);
 
-	connect(job, &KIO::PreviewJob::gotPreview, [=, f = std::move(f)](const KFileItem &, const QPixmap &image) mutable {
+	connect(job, &KIO::PreviewJob::gotPreview, job, [=, f = std::move(f)](const KFileItem &, const QPixmap &image) mutable {
 		QByteArray thumbnailData;
 		QBuffer thumbnailBuffer(&thumbnailData);
 		image.save(&thumbnailBuffer, "JPG", JPEG_EXPORT_QUALITY);
 
-		// Don't fill the ram just to find the dimensions of an image
-		if (file->size() < THUMBNAIL_GENERATION_MAX_FILE_SIZE) {
-			QImage image(file->fileName());
-
-			if (!image.isNull()) {
-				result->dimensions = image.size();
+		if (const QImageReader reader(file->fileName()); reader.canRead()) {
+			if (const QSize size = reader.size(); !size.isEmpty()) {
+				result->dimensions = size;
 			}
 		}
 
@@ -552,3 +596,33 @@ QFuture<std::shared_ptr<QXmppFileSharingManager::MetadataGeneratorResult>> Media
 
 	return interface.future();
 }
+
+QFuture<QByteArray> MediaUtils::generateThumbnail(const QUrl &localFileUrl, const QString &mimeTypeName, int edgePixelCount)
+{
+	QFutureInterface<QByteArray> interface;
+
+	KFileItemList items {
+		KFileItem {
+			localFileUrl,
+			mimeTypeName,
+		}
+	};
+
+	static auto allPlugins = KIO::PreviewJob::availablePlugins();
+
+	auto *job = new KIO::PreviewJob(items, QSize(edgePixelCount, edgePixelCount), &allPlugins);
+	job->setAutoDelete(true);
+
+	QObject::connect(job, &KIO::PreviewJob::gotPreview, [interface](auto, const QPixmap &preview) mutable {
+		reportFinishedResult(interface, MediaUtils::encodeImageThumbnail(preview));
+	});
+
+	QObject::connect(job, &KIO::PreviewJob::failed, [interface](const KFileItem &item) mutable {
+		qDebug() << "Could not generate a thumbnail for" << item.url();
+		reportFinishedResult(interface, {});
+	});
+
+	job->start();
+
+	return interface.future();
+}
