@@ -1,32 +1,9 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2019 Filipe Azevedo <pasnox@gmail.com>
+// SPDX-FileCopyrightText: 2020 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2022 Jonah Brüchert <jbb@kaidan.im>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
@@ -38,6 +15,7 @@
 #include <QXmppFileSharingManager.h>
 
 #include "Enums.h"
+#include "Globals.h"
 
 class MediaUtils : public QObject
 {
@@ -52,12 +30,54 @@ public:
 	Q_INVOKABLE static bool isGeoLocation(const QString &content);
 	Q_INVOKABLE static QGeoCoordinate locationCoordinate(const QString &content);
 	Q_INVOKABLE static bool localFileAvailable(const QString &filePath);
-	Q_INVOKABLE static QUrl fromLocalFile(const QString &filePath);
+
+	/**
+	 * Returns the URL of a copied file.
+	 */
+	Q_INVOKABLE static QUrl urlFromClipboard();
+
+	/**
+	 * Deletes a local file.
+	 */
+	Q_INVOKABLE static void deleteFile(const QUrl &url);
+
+	/**
+	 * Returns the path of a local file URL.
+	 */
+	Q_INVOKABLE static QString localFilePath(const QUrl &localFileUrl);
+
+	/**
+	 * Returns the URL of a local file.
+	 */
+	Q_INVOKABLE static QUrl localFileUrl(const QString &localFilePath);
+
+	/**
+	 * Returns the URL of a local file's directory.
+	 */
+	Q_INVOKABLE static QUrl localFileDirectoryUrl(const QUrl &localFileUrl);
+
+	Q_INVOKABLE static bool imageValid(const QImage &image);
+
 	Q_INVOKABLE static QMimeType mimeType(const QString &filePath);
 	Q_INVOKABLE static QMimeType mimeType(const QUrl &url);
 	Q_INVOKABLE static QString iconName(const QString &filePath);
 	Q_INVOKABLE static QString iconName(const QUrl &url);
 
+	/**
+	 * Returns the path of an audio file to be created.
+	 */
+	Q_INVOKABLE static QString newAudioFilePath();
+
+	/**
+	 * Returns the path of an image file to be created.
+	 */
+	Q_INVOKABLE static QString newImageFilePath();
+
+	/**
+	 * Returns the path of a video file to be created.
+	 */
+	Q_INVOKABLE static QString newVideoFilePath();
+
 	static QString mediaTypeName(Enums::MessageType mediaType);
 	Q_INVOKABLE static QString newMediaLabel(Enums::MessageType hint);
 	Q_INVOKABLE static QString newMediaIconName(Enums::MessageType hint);
@@ -82,6 +102,7 @@ public:
 	static QByteArray encodeImageThumbnail(const QImage &image);
 
 	static QFuture<std::shared_ptr<QXmppFileSharingManager::MetadataGeneratorResult>> generateMetadata(std::unique_ptr<QIODevice>);
+	static QFuture<QByteArray> generateThumbnail(const QUrl &localFileUrl, const QString &mimeTypeName, int edgePixelCount = THUMBNAIL_EDGE_PIXEL_COUNT);
 
 	static const QMimeDatabase &mimeDatabase()
 	{
