@@ -1,32 +1,10 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2019 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2019 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2019 caca hueto <cacahueto@olomono.de>
+// SPDX-FileCopyrightText: 2022 Bhavy Airi <airiragahv@gmail.com>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "Notifications.h"
 
@@ -36,9 +14,10 @@
 #endif
 
 // Kaidan
-#include "FutureUtils.h"
+#include "ChatController.h"
 #include "Kaidan.h"
-#include "MessageHandler.h"
+#include "MessageController.h"
+#include "RosterDb.h"
 #include "RosterModel.h"
 
 #include <QStringBuilder>
@@ -56,6 +35,7 @@ using namespace std::chrono_literals;
 // Event IDs corresponding to the section entries in the "kaidan.notifyrc" configuration file
 constexpr QStringView NEW_MESSAGE_EVENT_ID = u"new-message";
 constexpr QStringView NEW_SUBSEQUENT_MESSAGE_EVENT_ID = u"new-subsequent-message";
+constexpr QStringView PRESENCE_SUBSCRIPTION_REQUEST_EVENT_ID = u"presence-subscription-request";
 
 constexpr auto SUBSEQUENT_MESSAGE_INTERVAL = 5s;
 constexpr int MAXIMUM_NOTIFICATION_TEXT_LINE_COUNT = 6;
@@ -72,31 +52,27 @@ Notifications::Notifications(QObject *parent)
 {
 	Q_ASSERT(!s_instance);
 	s_instance = this;
-
-	connect(this, &Notifications::closeMessageNotificationRequested, this, &Notifications::closeMessageNotification);
 }
 
 #ifdef HAVE_KNOTIFICATIONS
 void Notifications::sendMessageNotification(const QString &accountJid, const QString &chatJid, const QString &messageId, const QString &messageBody)
 {
 #ifdef DESKTOP_LINUX_ALIKE_OS
-	static bool IS_USING_GNOME = qEnvironmentVariable("XDG_CURRENT_DESKTOP").contains("GNOME", Qt::CaseInsensitive);
+	static bool IS_USING_GNOME = qEnvironmentVariable("XDG_CURRENT_DESKTOP").contains(QStringLiteral("GNOME"), Qt::CaseInsensitive);
 #endif
 
 	KNotification *notification = nullptr;
-	QUuid notificationId;
 
-	auto notificationWrapperItr = std::find_if(m_openNotifications.begin(), m_openNotifications.end(), [&accountJid, &chatJid](const auto &notificationWrapper) {
+	auto notificationWrapperItr = std::find_if(m_openMessageNotifications.begin(), m_openMessageNotifications.end(), [&accountJid, &chatJid](const auto &notificationWrapper) {
 		return notificationWrapper.accountJid == accountJid && notificationWrapper.chatJid == chatJid;
 	});
 
 	// Update an existing notification or create a new one.
-	if (notificationWrapperItr != m_openNotifications.end()) {
+	if (notificationWrapperItr != m_openMessageNotifications.end()) {
 		auto &messages = notificationWrapperItr->messages;
 		messages.append(messageBody);
 
 		// Initialize variables by known values.
-		notificationId = notificationWrapperItr->id;
 		notificationWrapperItr->latestMessageId = messageId;
 
 		QList<QString> notificationTextLines;
@@ -154,11 +130,9 @@ void Notifications::sendMessageNotification(const QString &accountJid, const QSt
 		notificationWrapperItr->notification = notification;
 	} else {
 		notification = new KNotification(NEW_MESSAGE_EVENT_ID.toString());
-		notificationId = QUuid::createUuid();
 		notification->setText(messageBody);
 
-		NotificationWrapper notificationWrapper {
-			.id = notificationId,
+		MessageNotificationWrapper notificationWrapper {
 			.accountJid = accountJid,
 			.chatJid = chatJid,
 			.initalTimestamp = QDateTime::currentDateTimeUtc(),
@@ -166,14 +140,10 @@ void Notifications::sendMessageNotification(const QString &accountJid, const QSt
 			.messages = { messageBody },
 			.notification = notification
 		};
-		m_openNotifications.append(notificationWrapper);
+		m_openMessageNotifications.append(notificationWrapper);
 	}
 
-	// Use bare JID for users that are not present in our roster, so foreign users can't choose a
-	// name that looks like a known contact.
-	auto rosterItem = RosterModel::instance()->findItem(chatJid);
-	auto chatName = rosterItem ? rosterItem->displayName() : chatJid;
-	notification->setTitle(chatName);
+	notification->setTitle(determineChatName(chatJid));
 
 #ifdef DESKTOP_LINUX_ALIKE_OS
 	if (IS_USING_GNOME) {
@@ -183,36 +153,31 @@ void Notifications::sendMessageNotification(const QString &accountJid, const QSt
 #ifdef Q_OS_ANDROID
 	notification->setIconName("kaidan-bw");
 #endif
-	notification->setDefaultAction("Open");
-	notification->setActions({
-		QObject::tr("Mark as read")
-	});
+	notification->setDefaultAction(tr("Open"));
+	notification->setActions({ tr("Mark as read") });
 
-	QObject::connect(notification, &KNotification::defaultActivated, this, [=] {
-		emit Kaidan::instance()->openChatPageRequested(accountJid, chatJid);
-		emit Kaidan::instance()->raiseWindowRequested();
+	connect(notification, &KNotification::defaultActivated, this, [this, accountJid, chatJid] {
+		showChat(accountJid, chatJid);
 	});
-	QObject::connect(notification, &KNotification::action1Activated, this, [=] {
-		emit RosterModel::instance()->updateItemRequested(chatJid, [=](RosterItem &item) {
+	connect(notification, &KNotification::action1Activated, this, [=] {
+		RosterDb::instance()->updateItem(chatJid, [=](RosterItem &item) {
 			item.lastReadContactMessageId = messageId;
 			item.unreadMessages = 0;
 		});
 
 		if (const auto item = RosterModel::instance()->findItem(chatJid); item && item->readMarkerSendingEnabled) {
-			runOnThread(Kaidan::instance()->client()->messageHandler(), [chatJid, messageId]() {
-				Kaidan::instance()->client()->messageHandler()->sendReadMarker(chatJid, messageId);
-			});
+			MessageController::instance()->sendReadMarker(chatJid, messageId);
 		}
 	});
 
-	QObject::connect(notification, &KNotification::closed, this, [=, this]() {
-		auto notificationWrapperItr = std::find_if(m_openNotifications.begin(), m_openNotifications.end(), [accountJid, chatJid](const NotificationWrapper &notificationWrapper) {
+	connect(notification, &KNotification::closed, this, [=, this]() {
+		auto notificationWrapperItr = std::find_if(m_openMessageNotifications.begin(), m_openMessageNotifications.end(), [accountJid, chatJid](const MessageNotificationWrapper &notificationWrapper) {
 			return notificationWrapper.accountJid == accountJid && notificationWrapper.chatJid == chatJid;
 		});
 
-		if (notificationWrapperItr != m_openNotifications.end()) {
+		if (notificationWrapperItr != m_openMessageNotifications.end()) {
 			if (notificationWrapperItr->isDeletionEnabled) {
-				m_openNotifications.erase(notificationWrapperItr);
+				m_openMessageNotifications.erase(notificationWrapperItr);
 			} else {
 				notificationWrapperItr->isDeletionEnabled = true;
 			}
@@ -224,14 +189,95 @@ void Notifications::sendMessageNotification(const QString &accountJid, const QSt
 
 void Notifications::closeMessageNotification(const QString &accountJid, const QString &chatJid)
 {
-	auto notificationWrapperItr = std::find_if(m_openNotifications.begin(), m_openNotifications.end(), [accountJid, chatJid](const NotificationWrapper &notificationWrapper) {
+	const auto notificationWrapperItr = std::find_if(m_openMessageNotifications.cbegin(), m_openMessageNotifications.cend(), [accountJid, chatJid](const MessageNotificationWrapper &notificationWrapper) {
 		return notificationWrapper.accountJid == accountJid && notificationWrapper.chatJid == chatJid;
 	});
 
-	if (notificationWrapperItr != m_openNotifications.end()) {
-		m_openNotifications.erase(notificationWrapperItr);
+	if (notificationWrapperItr != m_openMessageNotifications.cend()) {
+		notificationWrapperItr->notification->close();
 	}
 }
+
+void Notifications::sendPresenceSubscriptionRequestNotification(const QString &accountJid, const QString &chatJid)
+{
+#ifdef DESKTOP_LINUX_ALIKE_OS
+	static bool IS_USING_GNOME = qEnvironmentVariable("XDG_CURRENT_DESKTOP").contains(QStringLiteral("GNOME"), Qt::CaseInsensitive);
+#endif
+
+	auto notificationWrapperItr = std::find_if(m_openMessageNotifications.begin(), m_openMessageNotifications.end(), [&accountJid, &chatJid](const auto &notificationWrapper) {
+		return notificationWrapper.accountJid == accountJid && notificationWrapper.chatJid == chatJid;
+	});
+
+	// Only create a new notification if none exists.
+	if (notificationWrapperItr != m_openMessageNotifications.end()) {
+		return;
+	}
+
+	KNotification *notification = new KNotification(PRESENCE_SUBSCRIPTION_REQUEST_EVENT_ID.toString());
+	notification->setTitle(determineChatName(chatJid));
+	notification->setText(tr("Requests to receive your personal data"));
+
+	PresenceSubscriptionRequestNotificationWrapper notificationWrapper {
+		.accountJid = accountJid,
+		.chatJid = chatJid,
+		.notification = notification
+	};
+	m_openPresenceSubscriptionRequestNotifications.append(notificationWrapper);
+
+#ifdef DESKTOP_LINUX_ALIKE_OS
+	if (IS_USING_GNOME) {
+		notification->setFlags(KNotification::Persistent);
+	}
+#endif
+#ifdef Q_OS_ANDROID
+	notification->setIconName("kaidan-bw");
+#endif
+
+	notification->setDefaultAction(tr("Open"));
+
+	connect(notification, &KNotification::defaultActivated, this, [this, accountJid, chatJid, notification] {
+		showChat(accountJid, chatJid);
+		notification->close();
+	});
+
+	connect(notification, &KNotification::closed, this, [=, this]() {
+		auto notificationWrapperItr = std::find_if(m_openPresenceSubscriptionRequestNotifications.begin(), m_openPresenceSubscriptionRequestNotifications.end(), [accountJid, chatJid](const PresenceSubscriptionRequestNotificationWrapper &notificationWrapper) {
+			return notificationWrapper.accountJid == accountJid && notificationWrapper.chatJid == chatJid;
+		});
+
+		if (notificationWrapperItr != m_openPresenceSubscriptionRequestNotifications.end()) {
+			m_openPresenceSubscriptionRequestNotifications.erase(notificationWrapperItr);
+		}
+	});
+
+	notification->sendEvent();
+}
+
+void Notifications::closePresenceSubscriptionRequestNotification(const QString &accountJid, const QString &chatJid)
+{
+	const auto notificationWrapperItr = std::find_if(m_openPresenceSubscriptionRequestNotifications.cbegin(), m_openPresenceSubscriptionRequestNotifications.cend(), [accountJid, chatJid](const PresenceSubscriptionRequestNotificationWrapper &notificationWrapper) {
+		return notificationWrapper.accountJid == accountJid && notificationWrapper.chatJid == chatJid;
+	});
+
+	if (notificationWrapperItr != m_openPresenceSubscriptionRequestNotifications.cend()) {
+		notificationWrapperItr->notification->close();
+	}
+}
+
+QString Notifications::determineChatName(const QString &chatJid) const
+{
+	auto rosterItem = RosterModel::instance()->findItem(chatJid);
+	return rosterItem ? rosterItem->displayName() : chatJid;
+}
+
+void Notifications::showChat(const QString &accountJid, const QString &chatJid)
+{
+	if (!ChatController::instance()->isChatCurrentChat(accountJid, chatJid)) {
+		Q_EMIT Kaidan::instance()->openChatPageRequested(accountJid, chatJid);
+	}
+
+	Q_EMIT Kaidan::instance()->raiseWindowRequested();
+}
 #else
 void Notifications::sendMessageNotification(const QString &, const QString &, const QString &, const QString &)
 {
@@ -240,4 +286,12 @@ void Notifications::sendMessageNotification(const QString &, const QString &, co
 void Notifications::closeMessageNotification(const QString &, const QString &)
 {
 }
+
+void Notifications::sendPresenceSubscriptionRequestNotification(const QString &accountJid, const QString &chatJid)
+{
+}
+
+void Notifications::closePresenceSubscriptionRequestNotification(const QString &accountJid, const QString &chatJid)
+{
+}
 #endif // HAVE_KNOTIFICATIONS
