@@ -1,35 +1,13 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2019 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2021 Jonah Brüchert <jbb@kaidan.im>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "RegistrationManager.h"
 
+// std
+#include <chrono>
 // QXmpp
 #include <QXmppBitsOfBinaryDataList.h>
 #include <QXmppRegistrationManager.h>
@@ -37,48 +15,78 @@
 #include "AccountManager.h"
 #include "BitsOfBinaryImageProvider.h"
 #include "ClientWorker.h"
-#include "Globals.h"
 #include "Kaidan.h"
 #include "RegistrationDataFormModel.h"
 #include "ServerFeaturesCache.h"
+#include "Settings.h"
+
+using namespace std::chrono_literals;
+
+constexpr auto ACCOUNT_DELETION_TIMEOUT = 5s;
 
 RegistrationManager::RegistrationManager(ClientWorker *clientWorker, QXmppClient *client, QObject *parent)
 	: QObject(parent),
 	  m_clientWorker(clientWorker),
 	  m_client(client),
-	  m_manager(new QXmppRegistrationManager),
+	  m_manager(client->addNewExtension<QXmppRegistrationManager>()),
 	  m_dataFormModel(new RegistrationDataFormModel())
 {
-	client->addExtension(m_manager);
-
 	connect(m_manager, &QXmppRegistrationManager::supportedByServerChanged, this, &RegistrationManager::handleInBandRegistrationSupportedChanged);
 
 	// account creation
-	connect(this, &RegistrationManager::sendRegistrationFormRequested,
-		this, &RegistrationManager::sendRegistrationForm);
+
+	connect(this, &RegistrationManager::registrationFormRequested, this, &RegistrationManager::requestRegistrationForm);
+	connect(this, &RegistrationManager::sendRegistrationFormRequested, this, &RegistrationManager::sendRegistrationForm);
+	connect(this, &RegistrationManager::abortRegistrationRequested, this, &RegistrationManager::abortRegistration);
 	connect(m_manager, &QXmppRegistrationManager::registrationFormReceived, this, &RegistrationManager::handleRegistrationFormReceived);
 	connect(m_manager, &QXmppRegistrationManager::registrationSucceeded, this, &RegistrationManager::handleRegistrationSucceeded);
 	connect(m_manager, &QXmppRegistrationManager::registrationFailed, this, &RegistrationManager::handleRegistrationFailed);
 
 	// account deletion
-	connect(m_manager, &QXmppRegistrationManager::accountDeletionFailed, m_clientWorker, &ClientWorker::handleAccountDeletionFromServerFailed);
-	connect(m_manager, &QXmppRegistrationManager::accountDeleted, m_clientWorker, &ClientWorker::handleAccountDeletedFromServer);
+	connect(m_manager, &QXmppRegistrationManager::accountDeleted, AccountManager::instance(), &AccountManager::handleAccountDeletedFromServer);
+	connect(m_manager, &QXmppRegistrationManager::accountDeletionFailed, AccountManager::instance(), &AccountManager::handleAccountDeletionFromServerFailed);
 
 	// password change
-	connect(this, &RegistrationManager::changePasswordRequested,
-		this, &RegistrationManager::changePassword);
+	connect(this, &RegistrationManager::changePasswordRequested, this, &RegistrationManager::changePassword);
 	connect(m_manager, &QXmppRegistrationManager::passwordChanged, this, &RegistrationManager::handlePasswordChanged);
 	connect(m_manager, &QXmppRegistrationManager::passwordChangeFailed, this, &RegistrationManager::handlePasswordChangeFailed);
 }
 
-void RegistrationManager::setRegisterOnConnectEnabled(bool registerOnConnect)
+bool RegistrationManager::registerOnConnectEnabled() const
 {
-	m_manager->setRegisterOnConnectEnabled(registerOnConnect);
+	return m_manager->registerOnConnectEnabled();
 }
 
 void RegistrationManager::deleteAccount()
 {
 	m_manager->deleteAccount();
+
+	// Start a timer to disconnect from the server after a specified timeout triggering
+	// QXmppRegistrationManager::accountDeleted to be emitted.
+	// Considering an account as deleted after a timeout is needed because some servers do not
+	// respond to the deletion request within a reasonable time period.
+	QTimer::singleShot(ACCOUNT_DELETION_TIMEOUT, m_client, &QXmppClient::disconnectFromServer);
+}
+
+void RegistrationManager::requestRegistrationForm()
+{
+	if (m_client->state() != QXmppClient::DisconnectedState) {
+		if (registerOnConnectEnabled() && m_client->configuration().jidBare() == AccountManager::instance()->jid()) {
+			m_manager->requestRegistrationForm();
+			return;
+		}
+
+		m_client->disconnectFromServer();
+	}
+
+	AccountManager::instance()->setHasNewCredentials(true);
+	setRegisterOnConnectEnabled(true);
+	m_clientWorker->connectToServer();
+}
+
+void RegistrationManager::setRegisterOnConnectEnabled(bool registerOnConnect)
+{
+	m_manager->setRegisterOnConnectEnabled(registerOnConnect);
 }
 
 void RegistrationManager::sendRegistrationForm()
@@ -94,7 +102,17 @@ void RegistrationManager::sendRegistrationForm()
 		m_manager->setRegistrationFormToSend(m_dataFormModel->form());
 	}
 
-	m_clientWorker->connectToRegister();
+	AccountManager::instance()->setHasNewCredentials(true);
+	setRegisterOnConnectEnabled(true);
+
+	// If the client is still connected/connecting to the server, send the registration form
+	// directly.
+	// Otherwise, reconnect to the server beforehand.
+	if (m_client->state() != QXmppClient::DisconnectedState) {
+		m_manager->sendCachedRegistrationForm();
+	} else {
+		m_clientWorker->connectToServer();
+	}
 }
 
 void RegistrationManager::changePassword(const QString &newPassword)
@@ -115,8 +133,6 @@ void RegistrationManager::handleInBandRegistrationSupportedChanged()
 
 void RegistrationManager::handleRegistrationFormReceived(const QXmppRegisterIq &iq)
 {
-	m_client->disconnectFromServer();
-
 	bool isFakeForm;
 	QXmppDataForm newDataForm = extractFormFromRegisterIq(iq, isFakeForm);
 
@@ -124,8 +140,8 @@ void RegistrationManager::handleRegistrationFormReceived(const QXmppRegisterIq &
 	if (newDataForm.fields().isEmpty()) {
 		// If there is a standardized out-of-band URL, use that.
 		if (!iq.outOfBandUrl().isEmpty()) {
-			emit Kaidan::instance()->registrationOutOfBandUrlReceived(iq.outOfBandUrl());
-			setRegisterOnConnectEnabled(false);
+			Q_EMIT Kaidan::instance()->registrationOutOfBandUrlReceived({ iq.outOfBandUrl() });
+			abortRegistration();
 			return;
 		}
 
@@ -134,23 +150,21 @@ void RegistrationManager::handleRegistrationFormReceived(const QXmppRegisterIq &
 		const auto words = iq.instructions().split(u' ');
 		for (const auto &instructionPart : words) {
 			if (instructionPart.startsWith(u"https://")) {
-				emit Kaidan::instance()->registrationOutOfBandUrlReceived(instructionPart);
-				setRegisterOnConnectEnabled(false);
+				Q_EMIT Kaidan::instance()->registrationOutOfBandUrlReceived({ instructionPart });
+				abortRegistration();
 				return;
 			}
 		}
 
-		// If no URL has been found in the instructions, there is a
-		// problem with the server.
-		emit m_clientWorker->connectionErrorChanged(ClientWorker::RegistrationUnsupported);
-		setRegisterOnConnectEnabled(false);
+		// If no URL has been found in the instructions, there is a problem with the server.
+		Q_EMIT m_clientWorker->connectionErrorChanged(ClientWorker::RegistrationUnsupported);
+		abortRegistration();
 		return;
 	}
 
 	copyUserDefinedValuesToNewForm(m_dataFormModel->form(), newDataForm);
-	cleanUpLastForm();
+	cleanUpLastForm(new RegistrationDataFormModel(newDataForm));
 
-	m_dataFormModel = new RegistrationDataFormModel(newDataForm);
 	m_dataFormModel->setIsFakeForm(isFakeForm);
 	// Move to the main thread, so QML can connect signals to the model.
 	m_dataFormModel->moveToThread(Kaidan::instance()->thread());
@@ -162,57 +176,70 @@ void RegistrationManager::handleRegistrationFormReceived(const QXmppRegisterIq &
 		m_contentIdsToRemove << bobData.cid();
 	}
 
-	emit Kaidan::instance()->registrationFormReceived(m_dataFormModel);
+	Q_EMIT Kaidan::instance()->registrationFormReceived(m_dataFormModel);
 }
 
 void RegistrationManager::handleRegistrationSucceeded()
 {
-	AccountManager::instance()->setJid(m_dataFormModel->extractUsername().append('@').append(m_client->configuration().domain()));
+	AccountManager::instance()->setJid(m_dataFormModel->extractUsername().append(QLatin1Char('@')).append(m_client->configuration().domain()));
 	AccountManager::instance()->setPassword(m_dataFormModel->extractPassword());
 
 	m_client->disconnectFromServer();
 	setRegisterOnConnectEnabled(false);
 	m_clientWorker->logIn();
 
-	cleanUpLastForm();
-	m_dataFormModel = new RegistrationDataFormModel();
+	cleanUpLastForm(new RegistrationDataFormModel());
 }
 
 void RegistrationManager::handleRegistrationFailed(const QXmppStanza::Error &error)
 {
-	m_client->disconnectFromServer();
-	setRegisterOnConnectEnabled(false);
-
 	RegistrationError registrationError = RegistrationError::UnknownError;
 
 	switch(error.type()) {
 	case QXmppStanza::Error::Cancel:
-		if (error.condition() == QXmppStanza::Error::FeatureNotImplemented)
+		if (error.condition() == QXmppStanza::Error::FeatureNotImplemented) {
 			registrationError = RegistrationError::InBandRegistrationNotSupported;
-		else if (error.condition() == QXmppStanza::Error::Conflict)
+			abortRegistration();
+		} else if (error.condition() == QXmppStanza::Error::Conflict) {
 			registrationError = RegistrationError::UsernameConflict;
-		else if (error.condition() == QXmppStanza::Error::NotAllowed && error.text().contains("captcha", Qt::CaseInsensitive))
+		} else if (error.condition() == QXmppStanza::Error::NotAllowed && error.text().contains(QStringLiteral("captcha"), Qt::CaseInsensitive)) {
 			registrationError = RegistrationError::CaptchaVerificationFailed;
+		}
+
 		break;
 	case QXmppStanza::Error::Modify:
 		if (error.condition() == QXmppStanza::Error::NotAcceptable) {
 			// TODO: Check error text in English (needs QXmpp change)
-			if (error.text().contains("password", Qt::CaseInsensitive) && (error.text().contains("weak", Qt::CaseInsensitive) || error.text().contains("short", Qt::CaseInsensitive)))
+			if (error.text().contains(QStringLiteral("password"), Qt::CaseInsensitive) && (error.text().contains(QStringLiteral("weak"), Qt::CaseInsensitive) || error.text().contains(QStringLiteral("short"), Qt::CaseInsensitive))) {
 				registrationError = RegistrationError::PasswordTooWeak;
-			else if (error.text().contains("ip", Qt::CaseInsensitive) || error.text().contains("quickly", Qt::CaseInsensitive)
-)
+			} else if (error.text().contains(QStringLiteral("ip"), Qt::CaseInsensitive) || error.text().contains(QStringLiteral("quickly"), Qt::CaseInsensitive)) {
 				registrationError = RegistrationError::TemporarilyBlocked;
-			else
+				abortRegistration();
+			} else {
 				registrationError = RegistrationError::RequiredInformationMissing;
-		} else if (error.condition() == QXmppStanza::Error::BadRequest && error.text().contains("captcha", Qt::CaseInsensitive)) {
+			}
+		} else if (error.condition() == QXmppStanza::Error::BadRequest && error.text().contains(QStringLiteral("captcha"), Qt::CaseInsensitive)) {
 			registrationError = RegistrationError::CaptchaVerificationFailed;
 		}
+
+		break;
+	case QXmppStanza::Error::Auth:
+		if (error.condition() == QXmppStanza::Error::Forbidden) {
+			registrationError = RegistrationError::InBandRegistrationNotSupported;
+			abortRegistration();
+		}
+		break;
+	case QXmppStanza::Error::Wait:
+		if (error.condition() == QXmppStanza::Error::ResourceConstraint || error.condition() == QXmppStanza::Error::PolicyViolation) {
+			registrationError = RegistrationError::TemporarilyBlocked;
+			abortRegistration();
+		}
 		break;
 	default:
 		break;
 	}
 
-	emit Kaidan::instance()->registrationFailed(quint8(registrationError), error.text());
+	Q_EMIT Kaidan::instance()->registrationFailed(quint8(registrationError), error.text());
 }
 
 void RegistrationManager::handlePasswordChanged(const QString &newPassword)
@@ -221,12 +248,12 @@ void RegistrationManager::handlePasswordChanged(const QString &newPassword)
 	AccountManager::instance()->storePassword();
 	AccountManager::instance()->setHasNewCredentials(false);
 	m_clientWorker->finishTask();
-	emit Kaidan::instance()->passwordChangeSucceeded();
+	Q_EMIT Kaidan::instance()->passwordChangeSucceeded();
 }
 
 void RegistrationManager::handlePasswordChangeFailed(const QXmppStanza::Error &error)
 {
-	emit Kaidan::instance()->passwordChangeFailed(error.text());
+	Q_EMIT Kaidan::instance()->passwordChangeFailed(error.text());
 	m_clientWorker->finishTask();
 }
 
@@ -289,9 +316,29 @@ void RegistrationManager::copyUserDefinedValuesToNewForm(const QXmppDataForm &ol
 	}
 }
 
-void RegistrationManager::cleanUpLastForm()
+void RegistrationManager::abortRegistration()
+{
+	const auto accountManager = AccountManager::instance();
+
+	// Reset the registration steps if the client connected to a server for registration.
+	if (accountManager->jid() != m_clientWorker->caches()->settings->authJid()) {
+		// Resetting the cached JID is needed to clear the JID field of LoginArea.
+		accountManager->setJid({});
+
+		// Disconnect from any server that the client is still connected to.
+		if (m_client->state() != QXmppClient::DisconnectedState) {
+			m_client->disconnectFromServer();
+		}
+
+		setRegisterOnConnectEnabled(false);
+		cleanUpLastForm(new RegistrationDataFormModel());
+	}
+}
+
+void RegistrationManager::cleanUpLastForm(RegistrationDataFormModel *newDataFormModel)
 {
 	delete m_dataFormModel;
+	m_dataFormModel = newDataFormModel;
 
 	// Remove content IDs from the last form.
 	for (auto itr = m_contentIdsToRemove.begin(); itr != m_contentIdsToRemove.end();) {
