@@ -1,38 +1,15 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2019 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2021 Jonah Brüchert <jbb@kaidan.im>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
 // Kaidan
 // Qt
 #include <QObject>
+#include <QTimer>
 #include <QVector>
 // QXmpp
 #include <QXmppBitsOfBinaryContentId.h>
@@ -65,26 +42,39 @@ public:
 	RegistrationManager(ClientWorker *clientWorker, QXmppClient *client, QObject *parent = nullptr);
 
 	/**
-	 * Sets whether a registration is requested for the next time when the client connects to the server.
-	 *
-	 * @param registerOnConnect true for requesting a registration on connecting, false otherwise
+	 * Connects to the server and requests a data form for account registration.
 	 */
-	void setRegisterOnConnectEnabled(bool registerOnConnect);
+	Q_SIGNAL void registrationFormRequested();
+
+	bool registerOnConnectEnabled() const;
+
+	/**
+	 * Emitted to send a completed data form for registration.
+	 */
+	Q_SIGNAL void sendRegistrationFormRequested();
+
+	/**
+	 * Aborts an ongoing registration.
+	 */
+	Q_SIGNAL void abortRegistrationRequested();
 
 	/**
 	 * Deletes the account from the server.
 	 */
 	void deleteAccount();
 
-signals:
-	void changePasswordRequested(const QString &newPassword);
+	Q_SIGNAL void changePasswordRequested(const QString &newPassword);
+
+private:
+	void requestRegistrationForm();
 
 	/**
-	 * Emitted to send a completed data form for registration.
+	 * Sets whether a registration is requested for the next time when the client connects to the server.
+	 *
+	 * @param registerOnConnect true for requesting a registration on connecting, false otherwise
 	 */
-	void sendRegistrationFormRequested();
+	void setRegisterOnConnectEnabled(bool registerOnConnect);
 
-private slots:
 	/**
 	 * Sends the form containing information to register an account.
 	 */
@@ -141,7 +131,6 @@ private slots:
 	 */
 	void handlePasswordChangeFailed(const QXmppStanza::Error &error);
 
-private:
 	/**
 	 * Extracts a form from an IQ stanza for registration.
 	 *
@@ -160,10 +149,15 @@ private:
 	 */
 	void copyUserDefinedValuesToNewForm(const QXmppDataForm &oldForm, QXmppDataForm &newForm);
 
+	/**
+	 * Aborts an ongoing registration.
+	 */
+	void abortRegistration();
+
 	/**
 	 * Cleans up the last form used for registration.
 	 */
-	void cleanUpLastForm();
+	void cleanUpLastForm(RegistrationDataFormModel *newDataFormModel);
 
 	ClientWorker *m_clientWorker;
 	QXmppClient *m_client;
