@@ -1,32 +1,11 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Mathis Brüchert <mbblp@protonmail.ch>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2022 Bhavy Airi <airiragahv@gmail.com>
+// SPDX-FileCopyrightText: 2023 Filipe Azevedo <pasnox@gmail.com>
+// SPDX-FileCopyrightText: 2023 Tibor Csötönyi <work@taibsu.de>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
@@ -48,16 +27,29 @@ class RosterModel : public QAbstractListModel
 {
 	Q_OBJECT
 
+	Q_PROPERTY(QStringList accountJids READ accountJids NOTIFY accountJidsChanged)
+	Q_PROPERTY(QStringList groups READ groups NOTIFY groupsChanged)
+
 public:
 	enum RosterItemRoles {
+		AccountJidRole,
 		JidRole,
 		NameRole,
-		LastExchangedRole,
+		GroupsRole,
+		IsGroupChatRole,
+		IsPublicGroupChatRole,
+		IsDeletedGroupChatRole,
+		LastMessageDateTimeRole,
 		UnreadMessagesRole,
 		LastMessageRole,
+		LastMessageIsDraftRole,
+		LastMessageIsOwnRole,
+		LastMessageGroupChatSenderNameRole,
 		PinnedRole,
-		DraftIdRole,
+		SelectedRole,
+		NotificationRuleRole,
 	};
+	Q_ENUM(RosterItemRoles)
 
 	/**
 	 * Result for adding a contact by an XMPP URI specifying how the URI is used
@@ -72,11 +64,11 @@ public:
 	static RosterModel *instance();
 
 	RosterModel(QObject *parent = nullptr);
+	~RosterModel() override;
 
-	Q_REQUIRED_RESULT bool isEmpty() const;
-	Q_REQUIRED_RESULT int rowCount(const QModelIndex &parent = QModelIndex()) const override;
-	Q_REQUIRED_RESULT QHash<int, QByteArray> roleNames() const override;
-	Q_REQUIRED_RESULT QVariant data(const QModelIndex &index, int role) const override;
+	[[nodiscard]] int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+	[[nodiscard]] QHash<int, QByteArray> roleNames() const override;
+	[[nodiscard]] QVariant data(const QModelIndex &index, int role) const override;
 
 	/**
 	 * Returns whether this model has a roster item with the passed properties.
@@ -87,6 +79,29 @@ public:
 	 */
 	Q_INVOKABLE bool hasItem(const QString &jid) const;
 
+	Q_SIGNAL void itemAdded(const QString &accountJid, const QString &jid);
+	Q_SIGNAL void itemAdditionFailed(const QString &accountJid, const QString &jid);
+	Q_SIGNAL void itemRemoved(const QString &accountJid, const QString &jid);
+
+	/**
+	 * Returns the account JIDs of all roster items.
+	 *
+	 * @return all account JIDs
+	 */
+	QStringList accountJids() const;
+	Q_SIGNAL void accountJidsChanged();
+
+	/**
+	 * Returns the roster groups of all roster items.
+	 *
+	 * @return all roster groups
+	 */
+	QStringList groups() const;
+	Q_SIGNAL void groupsChanged();
+
+	Q_INVOKABLE void updateGroup(const QString &oldGroup, const QString &newGroup);
+	Q_INVOKABLE void removeGroup(const QString &group);
+
 	/**
 	 * Returns whether an account's presence is subscribed by a roster item.
 	 *
@@ -107,11 +122,10 @@ public:
 	 *
 	 * @param uriString XMPP URI string that contains only a JID
 	 */
-	Q_INVOKABLE RosterModel::AddContactByUriResult addContactByUri(const QString &uriString);
+	Q_INVOKABLE RosterModel::AddContactByUriResult addContactByUri(const QString &accountJid, const QString &uriString);
 
 	QString lastReadOwnMessageId(const QString &accountJid, const QString &jid) const;
 	QString lastReadContactMessageId(const QString &accountJid, const QString &jid) const;
-	QString draftMessageId(const QString &accountJid, const QString &jid) const;
 
 	/**
 	 * Sends read markers for all roster items that have unsent (pending) ones.
@@ -129,61 +143,50 @@ public:
 	 */
 	std::optional<RosterItem> findItem(const QString &jid) const;
 
+	const QVector<RosterItem> &items() const;
+
 	Q_INVOKABLE void pinItem(const QString &accountJid, const QString &jid);
 	Q_INVOKABLE void unpinItem(const QString &accountJid, const QString &jid);
 	Q_INVOKABLE void reorderPinnedItem(const QString &accountJid, const QString &jid, int oldIndex, int newIndex);
 
+	Q_INVOKABLE void toggleSelected(const QString &accountJid, const QString &jid);
+	Q_INVOKABLE void resetSelected();
+
 	Q_INVOKABLE void setChatStateSendingEnabled(const QString &accountJid, const QString &jid, bool chatStateSendingEnabled);
 	Q_INVOKABLE void setReadMarkerSendingEnabled(const QString &accountJid, const QString &jid, bool readMarkerSendingEnabled);
+	Q_INVOKABLE void setNotificationRule(const QString &accountJid, const QString &jid, RosterItem::NotificationRule notificationRule);
+	Q_INVOKABLE void setAutomaticMediaDownloadsRule(const QString &accountJid, const QString &jid, RosterItem::AutomaticMediaDownloadsRule rule);
 
-signals:
-	void addItemRequested(const RosterItem &item);
-	void removeItemRequested(const QString &jid);
-	void updateItemRequested(const QString &jid,
-	                         const std::function<void (RosterItem &)> &updateItem);
-	void replaceItemsRequested(const QHash<QString, RosterItem> &items);
-
-	/**
-	 * Emitted to remove all roster items of an account or a specific roster item.
-	 *
-	 * @param accountJid JID of the account whose roster items are being removed
-	 * @param jid JID of the roster item being removed (optional)
-	 */
-	void removeItemsRequested(const QString &accountJid, const QString &jid = {});
-
-	/**
-	 * Emitted, whan a subscription request was received
-	 */
-	void subscriptionRequestReceived(const QString &from, const QString &msg);
-
-private slots:
+private:
 	void handleItemsFetched(const QVector<RosterItem> &items);
 
 	void addItem(const RosterItem &item);
-	void removeItem(const QString &jid);
-	void updateItem(const QString &jid,
-	                const std::function<void (RosterItem &)> &updateItem);
-	void replaceItems(const QHash<QString, RosterItem> &items);
+	void updateItem(const RosterItem &item);
+	void removeItem(const QString &accountJid, const QString &jid);
+	void removeItems(const QString &accountJid);
 
-	/**
-	 * Removes all roster items of an account or a specific roster item.
-	 *
-	 * @param accountJid JID of the account whose roster items are being removed
-	 * @param jid JID of the roster item being removed (optional)
-	 */
-	void removeItems(const QString &accountJid, const QString &jid = {});
+	void handleAccountChanged();
 
 	void handleMessageAdded(const Message &message, MessageOrigin origin);
+	void handleMessageUpdated(const Message &message);
 	void handleDraftMessageAdded(const Message &message);
 	void handleDraftMessageUpdated(const Message &message);
-	void handleDraftMessageRemoved(const QString &id);
-	void handleDraftMessageFetched(const Message &msg);
+	void handleDraftMessageRemoved(const Message &newLastMessage);
+	void handleMessageRemoved(const Message &newLastMessage);
 
-private:
+	QFuture<QVector<int>> updateLastMessage(QVector<RosterItem>::Iterator &itr,
+						   const Message &message,
+						   bool onlyUpdateIfNewerOrAtSameAge = true);
+
+	void updateOnDraftMessageChanged(QVector<RosterItem>::Iterator &itr);
+	int informAboutChangedData(QVector<RosterItem>::Iterator &itr, const QVector<int> &changedRoles);
 	void insertItem(int index, const RosterItem &item);
 	void updateItemPosition(int currentIndex);
 	int positionToAdd(const RosterItem &item);
 	int positionToMove(int currentIndex);
+	
+	QString formatLastMessageDateTime(const QDateTime &lastMessageDateTime) const;
+	QString determineGroupChatSenderName(const Message &message) const;
 
 	QVector<RosterItem> m_items;
 
