@@ -1,32 +1,9 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2019 Robert Maerkisch <zatroxde@protonmail.ch>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2023 Tibor Csötönyi <work@taibsu.de>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
@@ -96,7 +73,7 @@ public:
 	 */
 	bool executePendingAvatarChange();
 
-signals:
+Q_SIGNALS:
 	/**
 	 * Emitted when any vCard is received.
 	 *
@@ -109,7 +86,7 @@ signals:
 	void changeNicknameRequested(const QString &nickname);
 	void changeAvatarRequested(const QImage &avatar = {});
 
-private slots:
+private:
 	/**
 	 * Changes the user's nickname.
 	 *
@@ -118,7 +95,6 @@ private slots:
 	void changeNickname(const QString &nickname);
 	void changeAvatar(const QImage &avatar = {});
 
-private:
 	/**
 	 * Changes the nickname which was cached to be set after receiving the current vCard.
 	 */
