@@ -1,32 +1,16 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2016 geobra <s.g.b@gmx.de>
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2017 Ilya Bizyaev <bizyaev@zoho.com>
+// SPDX-FileCopyrightText: 2017 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2019 Filipe Azevedo <pasnox@gmail.com>
+// SPDX-FileCopyrightText: 2019 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2019 Robert Maerkisch <zatroxde@protonmail.ch>
+// SPDX-FileCopyrightText: 2020 Yury Gubich <blue@macaw.me>
+// SPDX-FileCopyrightText: 2022 Mathis Brüchert <mbb@kaidan.im>
+// SPDX-FileCopyrightText: 2023 Tibor Csötönyi <work@taibsu.de>
+// SPDX-FileCopyrightText: 2024 Filipe Azevedo <pasnox@gmail.com>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 // Qt
 #include <QCommandLineOption>
@@ -43,69 +27,91 @@
 // QXmpp
 #include <QXmppClient.h>
 #include <QXmppDiscoveryIq.h>
+#include <QXmppMixInfoItem.h>
+#include <QXmppMixParticipantItem.h>
 #include <QXmppVCardIq.h>
 #include <QXmppRegisterIq.h>
 #include <QXmppResultSet.h>
+#include <QXmppUri.h>
 #include <QXmppVersionIq.h>
 
-#include "qxmpp-exts/QXmppUri.h"
-
 // Kaidan
+#include "Account.h"
 #include "AccountManager.h"
+#include "AccountMigrationManager.h"
+#include "AccountQrCodeGenerator.h"
 #include "AudioDeviceModel.h"
+#include "AtmManager.h"
+#include "AuthenticatableEncryptionKeyModel.h"
+#include "AuthenticatedEncryptionKeyModel.h"
 #include "AvatarFileStorage.h"
 #include "BitsOfBinaryImageProvider.h"
+#include "Blocking.h"
 #include "CameraModel.h"
+#include "ChatController.h"
+#include "ChatHintModel.h"
+#include "ContactQrCodeGenerator.h"
 #include "CredentialsGenerator.h"
 #include "CredentialsValidator.h"
 #include "DataFormModel.h"
 #include "DiscoveryManager.h"
 #include "EmojiModel.h"
 #include "Encryption.h"
+#include "EncryptionController.h"
+#include "EncryptionWatcher.h"
 #include "Enums.h"
+#include "FileModel.h"
 #include "FileProgressCache.h"
+#include "FileProxyModel.h"
 #include "FileSharingController.h"
+#include "GroupChatController.h"
+#include "GroupChatQrCodeGenerator.h"
+#include "GroupChatUser.h"
+#include "GroupChatUserFilterModel.h"
+#include "GroupChatUserKeyAuthenticationFilterModel.h"
+#include "GroupChatUserModel.h"
 #include "GuiStyle.h"
+#include "HostCompletionModel.h"
+#include "HostCompletionProxyModel.h"
 #include "Kaidan.h"
-#include "MediaUtils.h"
+#include "LoginQrCodeGenerator.h"
 #include "MediaRecorder.h"
+#include "MediaUtils.h"
 #include "Message.h"
 #include "MessageComposition.h"
 #include "MessageModel.h"
-#include "MessageHandler.h"
-#include "OmemoManager.h"
-#include "OmemoWatcher.h"
+#include "MessageReactionModel.h"
+#include "ProviderListModel.h"
 #include "PublicGroupChatModel.h"
 #include "PublicGroupChatProxyModel.h"
 #include "PublicGroupChatSearchManager.h"
 #include "QmlUtils.h"
-#include "QrCodeGenerator.h"
 #include "QrCodeScannerFilter.h"
 #include "RegistrationDataFormFilterModel.h"
 #include "RegistrationManager.h"
+#include "RosterFilterProxyModel.h"
 #include "RosterItemWatcher.h"
 #include "RosterManager.h"
 #include "RosterModel.h"
-#include "RosterFilterProxyModel.h"
 #include "ServerFeaturesCache.h"
-#include "ProviderListModel.h"
 #include "StatusBar.h"
+#include "TextFormatter.h"
 #include "UserDevicesModel.h"
-#include "VCardModel.h"
 #include "VCardManager.h"
+#include "VCardModel.h"
 #include "VersionManager.h"
-#include "RecentPicturesModel.h"
-#include "NotificationsMutedWatcher.h"
+
+Q_DECLARE_ASSOCIATIVE_CONTAINER_METATYPE(QMultiHash)
 
 Q_DECLARE_METATYPE(Qt::ApplicationState)
 
 Q_DECLARE_METATYPE(QXmppClient::State)
 Q_DECLARE_METATYPE(QXmppMessage::State)
-Q_DECLARE_METATYPE(QXmppDiscoveryIq);
+Q_DECLARE_METATYPE(QXmppDiscoveryIq)
 Q_DECLARE_METATYPE(QXmppPresence)
 Q_DECLARE_METATYPE(QXmppStanza::Error)
-Q_DECLARE_METATYPE(QXmppResultSetReply);
-Q_DECLARE_METATYPE(QXmpp::TrustLevel);
+Q_DECLARE_METATYPE(QXmppResultSetReply)
+Q_DECLARE_METATYPE(QXmpp::TrustLevel)
 Q_DECLARE_METATYPE(QXmppUri)
 Q_DECLARE_METATYPE(QXmppVCardIq)
 Q_DECLARE_METATYPE(QXmppVersionIq)
@@ -113,10 +119,9 @@ Q_DECLARE_METATYPE(QXmppVersionIq)
 Q_DECLARE_METATYPE(std::function<void()>)
 Q_DECLARE_METATYPE(std::function<void(RosterItem&)>)
 Q_DECLARE_METATYPE(std::function<void(Message&)>)
+Q_DECLARE_METATYPE(std::function<void(GroupChatUser&)>)
 
-#ifdef STATIC_BUILD
-#include "static_plugins.h"
-#endif
+Q_DECLARE_METATYPE(std::shared_ptr<Message>)
 
 #ifndef QAPPLICATION_CLASS
 #define QAPPLICATION_CLASS QApplication
@@ -128,11 +133,6 @@ Q_DECLARE_METATYPE(std::function<void(Message&)>)
 #include "singleapp/singleapplication.h"
 #endif
 
-#ifdef STATIC_BUILD
-#define KIRIGAMI_BUILD_TYPE_STATIC
-#include "./3rdparty/kirigami/src/kirigamiplugin.h"
-#endif
-
 #ifdef Q_OS_ANDROID
 #include <QtAndroid>
 #endif
@@ -151,18 +151,18 @@ enum CommandLineParseResult {
 CommandLineParseResult parseCommandLine(QCommandLineParser &parser, QString *errorMessage)
 {
 	// application description
-	parser.setApplicationDescription(QString(APPLICATION_DISPLAY_NAME) +
-	                                 " - " + QString(APPLICATION_DESCRIPTION));
+	parser.setApplicationDescription(QStringLiteral(APPLICATION_DISPLAY_NAME) +
+	                                 QStringLiteral(" - ") + QStringLiteral(APPLICATION_DESCRIPTION));
 
 	// add all possible arguments
 	QCommandLineOption helpOption = parser.addHelpOption();
 	QCommandLineOption versionOption = parser.addVersionOption();
-	parser.addOption({"disable-xml-log", "Disable output of full XMPP XML stream."});
+	parser.addOption({QStringLiteral("disable-xml-log"), QStringLiteral("Disable output of full XMPP XML stream.")});
 #ifndef NDEBUG
-	parser.addOption({{"m", "multiple"}, "Allow multiple instances to be started."});
+	parser.addOption({{QStringLiteral("m"), QStringLiteral("multiple")}, QStringLiteral("Allow multiple instances to be started.")});
 #endif
-	parser.addPositionalArgument("xmpp-uri", "An XMPP-URI to open (i.e. join a chat).",
-	                             "[xmpp-uri]");
+	parser.addPositionalArgument(QStringLiteral("xmpp-uri"), QStringLiteral("An XMPP-URI to open (i.e. join a chat)."),
+	                             QStringLiteral("[xmpp-uri]"));
 
 	// parse arguments
 	if (!parser.parse(QGuiApplication::arguments())) {
@@ -203,10 +203,10 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 #endif
 
 	// name, display name, description
-	QGuiApplication::setApplicationName(APPLICATION_NAME);
-	QGuiApplication::setApplicationDisplayName(APPLICATION_DISPLAY_NAME);
-	QGuiApplication::setApplicationVersion(VERSION_STRING);
-	QGuiApplication::setDesktopFileName(APPLICATION_ID);
+	QGuiApplication::setApplicationName(QStringLiteral(APPLICATION_NAME));
+	QGuiApplication::setApplicationDisplayName(QStringLiteral(APPLICATION_DISPLAY_NAME));
+	QGuiApplication::setApplicationVersion(QStringLiteral(VERSION_STRING));
+	QGuiApplication::setDesktopFileName(QStringLiteral(APPLICATION_ID));
 	// attributes
 	QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 	QGuiApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
@@ -256,13 +256,14 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 	qRegisterMetaType<RosterManager*>();
 	qRegisterMetaType<Message>();
 	qRegisterMetaType<MessageModel*>();
-	qRegisterMetaType<MessageHandler*>();
+	qRegisterMetaType<ChatHintModel*>();
 	qRegisterMetaType<DiscoveryManager*>();
 	qRegisterMetaType<VCardManager*>();
 	qRegisterMetaType<VersionManager*>();
 	qRegisterMetaType<RegistrationManager*>();
+	qRegisterMetaType<AccountMigrationManager*>();
 	qRegisterMetaType<FileSharingController *>();
-	qRegisterMetaType<OmemoManager *>();
+	qRegisterMetaType<EncryptionController *>();
 	qRegisterMetaType<AvatarFileStorage*>();
 	qRegisterMetaType<QmlUtils*>();
 	qRegisterMetaType<QVector<Message>>();
@@ -283,6 +284,15 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 	qRegisterMetaType<QXmppVersionIq>();
 	qRegisterMetaType<QXmppUri>();
 	qRegisterMetaType<QMap<QString, QUrl>>();
+	qRegisterMetaType<std::shared_ptr<Message>>();
+	qRegisterMetaType<AtmManager*>();
+	qRegisterMetaType<GroupChatController*>();
+	qRegisterMetaType<GroupChatUser>();
+	qRegisterMetaType<QVector<GroupChatUser>>();
+	qRegisterMetaType<std::function<void(GroupChatUser&)>>();
+	qRegisterMetaType<GroupChatUserModel*>();
+	// The alias is needed because the type shares its QMetaType::id() with quint32.
+	qRegisterMetaType<uint32_t>("uint32_t");
 
 	// Enums for c++ member calls using enums
 	qRegisterMetaType<Qt::ApplicationState>();
@@ -308,6 +318,18 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 	qRegisterMetaType<ProviderListModel::Role>();
 	qRegisterMetaType<ChatState::State>();
 	qRegisterMetaType<Encryption>();
+	qRegisterMetaType<MessageReactionDeliveryState>();
+	qRegisterMetaType<FileModel::Role>();
+	qRegisterMetaType<FileProxyModel::Mode>();
+	qRegisterMetaType<Account::ContactNotificationRule>();
+	qRegisterMetaType<Account::GroupChatNotificationRule>();
+	qRegisterMetaType<Account::AutomaticMediaDownloadsRule>();
+	qRegisterMetaType<Account::GeoLocationMapService>();
+	qRegisterMetaType<RosterItem::AutomaticMediaDownloadsRule>();
+	qRegisterMetaType<AccountMigrationManager::MigrationState>();
+	qRegisterMetaType<RosterItem::GroupChatFlag>();
+	qRegisterMetaType<RosterModel::RosterItemRoles>();
+	qRegisterMetaType<Message::TrustLevel>();
 
 	// QXmpp
 	qRegisterMetaType<QXmppResultSetReply>();
@@ -315,10 +337,13 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 	qRegisterMetaType<QXmppPresence>();
 	qRegisterMetaType<QXmppDiscoveryIq>();
 	qRegisterMetaType<QHash<QString, QHash<QByteArray, QXmpp::TrustLevel>>>();
+	qRegisterMetaType<QXmppMixInfoItem>();
+	qRegisterMetaType<QXmppMixParticipantItem>();
+	qRegisterMetaType<QMultiHash<QString, QByteArray>>();
 
 	// Qt-Translator
 	QTranslator qtTranslator;
-	qtTranslator.load("qt_" + QLocale::system().name(),
+	qtTranslator.load(QStringLiteral("qt_") + QLocale::system().name(),
 	                  QLibraryInfo::location(QLibraryInfo::TranslationsPath));
 	QCoreApplication::installTranslator(&qtTranslator);
 
@@ -350,9 +375,9 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 		qDebug() << "Another instance of" << APPLICATION_DISPLAY_NAME << "is already running!";
 #else
 	// check if another instance already runs
-	if (app.isSecondary() && !parser.isSet("multiple")) {
-		qDebug().noquote() << QString("Another instance of %1 is already running.")
-		                      .arg(APPLICATION_DISPLAY_NAME)
+	if (app.isSecondary() && !parser.isSet(QStringLiteral("multiple"))) {
+		qDebug().noquote() << QStringLiteral("Another instance of %1 is already running.")
+		                      .arg(QStringLiteral(APPLICATION_DISPLAY_NAME))
 		                   << "You can enable multiple instances by specifying '--multiple'.";
 #endif
 
@@ -366,7 +391,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 	//
 	// Kaidan back-end
 	//
-	Kaidan kaidan(!parser.isSet("disable-xml-log"));
+	Kaidan kaidan(!parser.isSet(QStringLiteral("disable-xml-log")));
 
 #if !defined(Q_OS_IOS) && !defined(Q_OS_ANDROID)
 	// receive messages from other instances of Kaidan
@@ -383,7 +408,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 	//
 
 	if (QIcon::themeName().isEmpty()) {
-		QIcon::setThemeName("breeze");
+		QIcon::setThemeName(QStringLiteral("breeze"));
 	}
 
 	QQmlApplicationEngine engine;
@@ -406,19 +431,22 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 #endif
 
 	// QML type bindings
-#ifdef STATIC_BUILD
-	KirigamiPlugin::getInstance().registerTypes();
-#endif
+	qmlRegisterType<AccountQrCodeGenerator>(APPLICATION_ID, 1, 0, "AccountQrCodeGenerator");
 	qmlRegisterType<StatusBar>("StatusBar", 0, 1, "StatusBar");
 	qmlRegisterType<EmojiModel>("EmojiModel", 0, 1, "EmojiModel");
 	qmlRegisterType<EmojiProxyModel>("EmojiModel", 0, 1, "EmojiProxyModel");
 	qmlRegisterType<QrCodeScannerFilter>(APPLICATION_ID, 1, 0, "QrCodeScannerFilter");
 	qmlRegisterType<VCardModel>(APPLICATION_ID, 1, 0, "VCardModel");
 	qmlRegisterType<RosterFilterProxyModel>(APPLICATION_ID, 1, 0, "RosterFilterProxyModel");
+	qmlRegisterType<BlockingModel>(APPLICATION_ID, 1, 0, "BlockingModel");
+	qmlRegisterType<BlockingWatcher>(APPLICATION_ID, 1, 0, "BlockingWatcher");
+	qmlRegisterType<BlockingAction>(APPLICATION_ID, 1, 0, "BlockingAction");
+	qmlRegisterType<ContactQrCodeGenerator>(APPLICATION_ID, 1, 0, "ContactQrCodeGenerator");
 	qmlRegisterType<MessageComposition>(APPLICATION_ID, 1, 0, "MessageComposition");
 	qmlRegisterType<FileSelectionModel>(APPLICATION_ID, 1, 0, "FileSelectionModel");
 	qmlRegisterType<CameraModel>(APPLICATION_ID, 1, 0, "CameraModel");
 	qmlRegisterType<AudioDeviceModel>(APPLICATION_ID, 1, 0, "AudioDeviceModel");
+	qmlRegisterType<LoginQrCodeGenerator>(APPLICATION_ID, 1, 0, "LoginQrCodeGenerator");
 	qmlRegisterType<MediaSettingsContainerModel>(APPLICATION_ID, 1, 0, "MediaSettingsContainerModel");
 	qmlRegisterType<MediaSettingsResolutionModel>(APPLICATION_ID, 1, 0, "MediaSettingsResolutionModel");
 	qmlRegisterType<MediaSettingsQualityModel>(APPLICATION_ID, 1, 0, "MediaSettingsQualityModel");
@@ -431,51 +459,68 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 	qmlRegisterType<UserDevicesModel>(APPLICATION_ID, 1, 0, "UserDevicesModel");
 	qmlRegisterType<CredentialsGenerator>(APPLICATION_ID, 1, 0, "CredentialsGenerator");
 	qmlRegisterType<CredentialsValidator>(APPLICATION_ID, 1, 0, "CredentialsValidator");
-	qmlRegisterType<QrCodeGenerator>(APPLICATION_ID, 1, 0, "QrCodeGenerator");
 	qmlRegisterType<RegistrationDataFormFilterModel>(APPLICATION_ID, 1, 0, "RegistrationDataFormFilterModel");
 	qmlRegisterType<ProviderListModel>(APPLICATION_ID, 1, 0, "ProviderListModel");
 	qmlRegisterType<FileProgressWatcher>(APPLICATION_ID, 1, 0, "FileProgressWatcher");
 	qmlRegisterType<UserPresenceWatcher>(APPLICATION_ID, 1, 0, "UserPresenceWatcher");
 	qmlRegisterType<UserResourcesWatcher>(APPLICATION_ID, 1, 0, "UserResourcesWatcher");
 	qmlRegisterType<RosterItemWatcher>(APPLICATION_ID, 1, 0, "RosterItemWatcher");
-	qmlRegisterType<RecentPicturesModel>(APPLICATION_ID, 1, 0, "RecentPicturesModel");
-	qmlRegisterType<NotificationsMutedWatcher>(APPLICATION_ID, 1, 0, "NotificationsMutedWatcher");
 	qmlRegisterType<PublicGroupChatSearchManager>("PublicGroupChats", 1, 0, "SearchManager");
 	qmlRegisterType<PublicGroupChatModel>("PublicGroupChats", 1, 0, "Model");
 	qmlRegisterType<PublicGroupChatProxyModel>("PublicGroupChats", 1, 0, "ProxyModel");
-	qmlRegisterType<OmemoWatcher>(APPLICATION_ID, 1, 0, "OmemoWatcher");
+	qmlRegisterType<AuthenticatableEncryptionKeyModel>(APPLICATION_ID, 1, 0, "AuthenticatableEncryptionKeyModel");
+	qmlRegisterType<AuthenticatedEncryptionKeyModel>(APPLICATION_ID, 1, 0, "AuthenticatedEncryptionKeyModel");
+	qmlRegisterType<EncryptionWatcher>(APPLICATION_ID, 1, 0, "EncryptionWatcher");
+	qmlRegisterType<HostCompletionModel>(APPLICATION_ID, 1, 0, "HostCompletionModel");
+	qmlRegisterType<HostCompletionProxyModel>(APPLICATION_ID, 1, 0, "HostCompletionProxyModel");
+	qmlRegisterType<FileModel>(APPLICATION_ID, 1, 0, "FileModel");
+	qmlRegisterType<FileProxyModel>(APPLICATION_ID, 1, 0, "FileProxyModel");
+	qmlRegisterType<TextFormatter>(APPLICATION_ID, 1, 0, "TextFormatter");
+	qmlRegisterType<GroupChatQrCodeGenerator>(APPLICATION_ID, 1, 0, "GroupChatQrCodeGenerator");
+	qmlRegisterType<GroupChatUserModel>(APPLICATION_ID, 1, 0, "GroupChatUserModel");
+	qmlRegisterType<GroupChatUserFilterModel>(APPLICATION_ID, 1, 0, "GroupChatUserFilterModel");
+	qmlRegisterType<GroupChatUserKeyAuthenticationFilterModel>(APPLICATION_ID, 1, 0, "GroupChatUserKeyAuthenticationFilterModel");
+	qmlRegisterType<AccountTrustMessageUriGenerator>(APPLICATION_ID, 1, 0, "AccountTrustMessageUriGenerator");
+	qmlRegisterType<ContactTrustMessageUriGenerator>(APPLICATION_ID, 1, 0, "ContactTrustMessageUriGenerator");
+	qmlRegisterType<MessageReactionModel>(APPLICATION_ID, 1, 0, "MessageReactionModel");
 
-	qmlRegisterUncreatableType<QAbstractItemModel>("EmojiModel", 0, 1, "QAbstractItemModel", "Used by proxy models");
-	qmlRegisterUncreatableType<Emoji>("EmojiModel", 0, 1, "Emoji", "Used by emoji models");
-	qmlRegisterUncreatableType<QMimeType>(APPLICATION_ID, 1, 0, "QMimeType", "QMimeType type usable");
-	qmlRegisterUncreatableType<CameraInfo>(APPLICATION_ID, 1, 0, "CameraInfo", "CameraInfo type usable");
-	qmlRegisterUncreatableType<AudioDeviceInfo>(APPLICATION_ID, 1, 0, "AudioDeviceInfo", "AudioDeviceInfo type usable");
-	qmlRegisterUncreatableType<MediaSettings>(APPLICATION_ID, 1, 0, "MediaSettings", "MediaSettings type usable");
-	qmlRegisterUncreatableType<CommonEncoderSettings>(APPLICATION_ID, 1, 0, "CommonEncoderSettings", "CommonEncoderSettings type usable");
-	qmlRegisterUncreatableType<ImageEncoderSettings>(APPLICATION_ID, 1, 0, "ImageEncoderSettings", "ImageEncoderSettings type usable");
-	qmlRegisterUncreatableType<AudioEncoderSettings>(APPLICATION_ID, 1, 0, "AudioEncoderSettings", "AudioEncoderSettings type usable");
-	qmlRegisterUncreatableType<VideoEncoderSettings>(APPLICATION_ID, 1, 0, "VideoEncoderSettings", "VideoEncoderSettings type usable");
-	qmlRegisterUncreatableType<ClientWorker>(APPLICATION_ID, 1, 0, "ClientWorker", "Cannot create object; only enums defined!");
-	qmlRegisterUncreatableType<DataFormModel>(APPLICATION_ID, 1, 0, "DataFormModel", "Cannot create object; only enums defined!");
-	qmlRegisterUncreatableType<Presence>(APPLICATION_ID, 1, 0, "Presence", "Cannot create object; only enums defined!");
-	qmlRegisterUncreatableType<RegistrationManager>(APPLICATION_ID, 1, 0, "RegistrationManager", "Cannot create object; only enums defined!");
-	qmlRegisterUncreatableType<ChatState>(APPLICATION_ID, 1, 0, "ChatState", "Cannot create object; only enums defined");
-	qmlRegisterUncreatableType<RosterModel>(APPLICATION_ID, 1, 0, "RosterModel", "Cannot create object; only enums defined!");
-	qmlRegisterUncreatableType<ServerFeaturesCache>(APPLICATION_ID, 1, 0, "ServerFeaturesCache", "ServerFeaturesCache type usable");
-	qmlRegisterUncreatableType<Encryption>(APPLICATION_ID, 1, 0, "Encryption", "Cannot create object; only enums defined!");
-	qmlRegisterUncreatableType<File>(APPLICATION_ID, 1, 0, "File", "Not creatable from QML");
-	qmlRegisterUncreatableType<PublicGroupChat>("PublicGroupChats", 1, 0, "PublicGroupChat", "Used by PublicGroupChatModel");
+	qmlRegisterUncreatableType<Account>(APPLICATION_ID, 1, 0, "Account", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<QAbstractItemModel>("EmojiModel", 0, 1, "QAbstractItemModel", QStringLiteral("Used by proxy models"));
+	qmlRegisterUncreatableType<Emoji>("EmojiModel", 0, 1, "Emoji", QStringLiteral("Used by emoji models"));
+	qmlRegisterUncreatableType<QMimeType>(APPLICATION_ID, 1, 0, "QMimeType", QStringLiteral("QMimeType type usable"));
+	qmlRegisterUncreatableType<CameraInfo>(APPLICATION_ID, 1, 0, "CameraInfo", QStringLiteral("CameraInfo type usable"));
+	qmlRegisterUncreatableType<AudioDeviceInfo>(APPLICATION_ID, 1, 0, "AudioDeviceInfo", QStringLiteral("AudioDeviceInfo type usable"));
+	qmlRegisterUncreatableType<MediaSettings>(APPLICATION_ID, 1, 0, "MediaSettings", QStringLiteral("MediaSettings type usable"));
+	qmlRegisterUncreatableType<CommonEncoderSettings>(APPLICATION_ID, 1, 0, "CommonEncoderSettings", QStringLiteral("CommonEncoderSettings type usable"));
+	qmlRegisterUncreatableType<ImageEncoderSettings>(APPLICATION_ID, 1, 0, "ImageEncoderSettings", QStringLiteral("ImageEncoderSettings type usable"));
+	qmlRegisterUncreatableType<AudioEncoderSettings>(APPLICATION_ID, 1, 0, "AudioEncoderSettings", QStringLiteral("AudioEncoderSettings type usable"));
+	qmlRegisterUncreatableType<VideoEncoderSettings>(APPLICATION_ID, 1, 0, "VideoEncoderSettings", QStringLiteral("VideoEncoderSettings type usable"));
+	qmlRegisterUncreatableType<ClientWorker>(APPLICATION_ID, 1, 0, "ClientWorker", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<DataFormModel>(APPLICATION_ID, 1, 0, "DataFormModel", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<Presence>(APPLICATION_ID, 1, 0, "Presence", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<RegistrationManager>(APPLICATION_ID, 1, 0, "RegistrationManager", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<AccountMigrationManager>(APPLICATION_ID, 1, 0, "AccountMigrationManager", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<ChatState>(APPLICATION_ID, 1, 0, "ChatState", QStringLiteral("Cannot create object; only enums defined"));
+	qmlRegisterUncreatableType<RosterModel>(APPLICATION_ID, 1, 0, "RosterModel", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<ServerFeaturesCache>(APPLICATION_ID, 1, 0, "ServerFeaturesCache", QStringLiteral("ServerFeaturesCache type usable"));
+	qmlRegisterUncreatableType<Encryption>(APPLICATION_ID, 1, 0, "Encryption", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<File>(APPLICATION_ID, 1, 0, "File", QStringLiteral("Not creatable from QML"));
+	qmlRegisterUncreatableType<PublicGroupChat>("PublicGroupChats", 1, 0, "PublicGroupChat", QStringLiteral("Used by PublicGroupChatModel"));
+	qmlRegisterUncreatableType<HostCompletionModel>(APPLICATION_ID, 1, 0, "HostCompletionModel", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<MessageReactionDeliveryState>(APPLICATION_ID, 1, 0, "MessageReactionDeliveryState", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<RosterItem>(APPLICATION_ID, 1, 0, "RosterItem", QStringLiteral("Cannot create object; only enums defined!"));
+	qmlRegisterUncreatableType<Message>(APPLICATION_ID, 1, 0, "Message", QStringLiteral("Cannot create object; only enums defined!"));
 
-	qmlRegisterUncreatableMetaObject(ChatState::staticMetaObject, APPLICATION_ID, 1, 0, "ChatState", "Can't create object; only enums defined!");
-	qmlRegisterUncreatableMetaObject(Enums::staticMetaObject, APPLICATION_ID, 1, 0, "Enums", "Can't create object; only enums defined!");
+	qmlRegisterUncreatableMetaObject(ChatState::staticMetaObject, APPLICATION_ID, 1, 0, "ChatState", QStringLiteral("Can't create object; only enums defined!"));
+	qmlRegisterUncreatableMetaObject(Enums::staticMetaObject, APPLICATION_ID, 1, 0, "Enums", QStringLiteral("Can't create object; only enums defined!"));
 
-	qmlRegisterSingletonType<MediaUtils>("MediaUtils", 0, 1, "MediaUtilsInstance", [](QQmlEngine *, QJSEngine *) {
-		QObject *instance = new MediaUtils(qApp);
-		return instance;
-	});
 	qmlRegisterSingletonType<QmlUtils>(APPLICATION_ID, 1, 0, "Utils", [](QQmlEngine *, QJSEngine *) {
 		return static_cast<QObject*>(QmlUtils::instance());
 	});
+	qmlRegisterSingletonType<MediaUtils>(APPLICATION_ID, 1, 0, "MediaUtils", [](QQmlEngine *, QJSEngine *) {
+		QObject *instance = new MediaUtils(qApp);
+		return instance;
+	});
 	qmlRegisterSingletonType<Kaidan>(APPLICATION_ID, 1, 0, "Kaidan", [](QQmlEngine *engine, QJSEngine *) {
 		engine->setObjectOwnership(Kaidan::instance(), QQmlEngine::CppOwnership);
 		return static_cast<QObject *>(Kaidan::instance());
@@ -486,14 +531,30 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
 	qmlRegisterSingletonType<AccountManager>(APPLICATION_ID, 1, 0, "AccountManager", [](QQmlEngine *, QJSEngine *) {
 		return static_cast<QObject *>(AccountManager::instance());
 	});
+	qmlRegisterSingletonType<ChatController>(APPLICATION_ID, 1, 0, "ChatController", [](QQmlEngine *, QJSEngine *) {
+		return static_cast<QObject *>(ChatController::instance());
+	});
+	qmlRegisterSingletonType<GroupChatController>(APPLICATION_ID, 1, 0, "GroupChatController", [](QQmlEngine *, QJSEngine *) {
+		return static_cast<QObject *>(GroupChatController::instance());
+	});
+	qmlRegisterSingletonType<EncryptionController>(APPLICATION_ID, 1, 0, "EncryptionController", [](QQmlEngine *, QJSEngine *) {
+		return static_cast<QObject *>(EncryptionController::instance());
+	});
 	qmlRegisterSingletonType<RosterModel>(APPLICATION_ID, 1, 0, "RosterModel", [](QQmlEngine *, QJSEngine *) {
 		return static_cast<QObject *>(RosterModel::instance());
 	});
 	qmlRegisterSingletonType<MessageModel>(APPLICATION_ID, 1, 0, "MessageModel", [](QQmlEngine *, QJSEngine *) {
 		return static_cast<QObject *>(MessageModel::instance());
 	});
+	qmlRegisterSingletonType<ChatHintModel>(APPLICATION_ID, 1, 0, "ChatHintModel", [](QQmlEngine *, QJSEngine *) {
+		return static_cast<QObject *>(ChatHintModel::instance());
+	});
+	qmlRegisterSingletonType<HostCompletionModel>(APPLICATION_ID, 1, 0, "HostCompletionModel", [](QQmlEngine *, QJSEngine *) {
+		static auto self = new HostCompletionModel(qApp);
+		return static_cast<QObject *>(self);
+	});
 
-	engine.load(QUrl("qrc:/qml/main.qml"));
+	engine.load(QUrl(QStringLiteral("qrc:/qml/main.qml")));
 	if (engine.rootObjects().isEmpty())
 		return -1;
 
